import flux_mcp.job as job
import flux_mcp.sched as sched
import flux_mcp.transformer as transform
import flux_mcp.validate as validate

TOOLS = [
    # Validation and counting
    validate.flux_validate_jobspec,
    validate.flux_count_jobspec_resources,
    validate.flux_validate_jobspec_persona,
    # Job functions
    job.flux_handle_delegation,
    transform.transform_jobspec,
    transform.transform_jobspec_persona,
    # Job core
    job.flux_submit_job,
    job.flux_cancel_job,
    job.flux_get_job_info,
    job.flux_get_job_logs,
    # Flux sched
    sched.flux_sched_init_graph,
    sched.flux_sched_partial_cancel,
    sched.flux_sched_cancel_job,
    sched.flux_sched_job_info,
    sched.flux_sched_match_allocate,
]
